/**
    Объект Application.
    Основной объект приложения
**/
var SmotreshkaApp = function() {
};

// prototype
SmotreshkaApp.prototype = {
    // текущая вьюха
    baseView: null,

    // ID DOM элементов
    IDS: {
        contentID: "main"
    },

    // список доступных роутингов
    ROUTES: {
        "play": {
            "fnc": "onPlay"
        },
        "login": {
            "fnc": "onLogin"
        },
    },

    //
    onBefore: function() {
        location.hash = "#";

        // инициализируем специфический для текущий платформы код
        SmotreshkaCore.SmartTV.Manager.initialize();

        // инициирует работу с клавиатурой
        SmotreshkaCore.SmartTV.Keyboard.initialize();

        //
        SmotreshkaCore.SmartTV.Unload.initialize();

        // переопределяем данные модели
        SmotreshkaCore.Store.Models.Model.prototype.baseUrl = "http://"+SmotreshkaCore.Config.Host;

        //
        SmotreshkaCore.Store.Models.VodMovie.prototype.baseUrl = "http://"+SmotreshkaCore.Config.VodHost;

        // включаем withCredentials
        SmotreshkaCore.Store.Models.Model.prototype.settings.withCredentials = true;

        //
        SmotreshkaCore.Store.Models.Model.prototype.settings.language = SmotreshkaCore.Translate.Helper.getCurrentLang().id.toLowerCase();
    },

    //
    onReady: function() {
        /** Добавляем в Body класс с названием платформы **/
        document.body.classList.add(SmotreshkaCore.PlatformDetector.currentPlatform.name.toLowerCase());

        /** Добавляем в Body класс с названием браузера **/
        document.body.classList.add(SmotreshkaCore.PlatformDetector.browser.toLowerCase());

        // включаем настройки миграции fe
        SmotreshkaCore.Store.Migrate.initialize(SmotreshkaCore.App.applicationInfo.model.version);

        // блок с контентом
        this.content = document.getElementById(this.IDS.contentID);

        // инициализируем работу с onscreen клавиатуру
        this.virtualKeyboard = new SmotreshkaCore.SmartTV.VirtualKeyboard();

        // слушаем изменение хеша браузера
        window.addEventListener("hashchange", this.onHashChange.bind(this), false);

        //
        SmotreshkaCore.Store.TokenManager.initialize("session");

        // модель аккаунта пользователя
        this.account = new SmotreshkaCore.Store.Models.Account();

        // успешно получили аккаунт
        this.account.on("sync", this.userLoaded.bind(this));

        // ошибка получения аккаунта
        this.account.on("error", this.userFail.bind(this));

        // коллекция все каналов
        this.channelCollection = new SmotreshkaCore.Store.Collections.Channels();

        // коллекция жанров
        this.genresCollection = new SmotreshkaCore.Store.Collections.Genre();

        // коллекция EPG
        this.epgCollection = new SmotreshkaCore.Store.Collections.EPG();

        //
        this.approveAcoount = new SmotreshkaCore.Store.Models.ApproveAccount();  
        this.approveAcoount.on("sync", this.approveAcoountSync.bind(this));
        this.approveAcoount.on("error", this.approveAcoountError.bind(this));

        // получаем с сервера информацию про аккаунт
        this.account.fetch();
    },

    // изменился хеш
    onHashChange: function() {
        var hash = location.hash.replace("#", "");

        this.disposeCurrentView();

        // перебираем доступные роуты
        for(var key in this.ROUTES) {
            if (this.ROUTES.hasOwnProperty(key) && key == hash) {
                // вызываем функцию для нужного роутинга
                this[this.ROUTES[key].fnc]();

                break;
            }
        }
    },

    // пользователь загрузился
    userLoaded: function(data, xhr, er) {
        var routePath = this.account.model.anonymous === true ? "#login" : "#play";
 
        if(this.account.model.anonymous !== true) {
            this.approve(routePath);
        } else {
            location.hash = routePath;
        }
    },

    /**
     * 
     */
    approveAcoountSync: function(data) {
        var isScripts = false;

        if(data) {
            SmotreshkaCore.App.approvedState = data.status;

            if(data.scripts) {
                isScripts = true;

                SmotreshkaCore.Dom.Helper.addScripts(data.scripts, function() {});
            }
            
        }

        if(!isScripts) {
            location.hash = "#play";
        }
    },

    /**
     * 
     */
    approveAcoountError: function() {
        location.hash = "#play";
    },

    /**
     * 
     */
    approve: function() {
        this.approveAcoount.updateAccount(this.account.model.user.id);
        this.approveAcoount.fetch();
    },

    // ошибка загрузки пользователя
    userFail: function(error, xhr) {
        var routePath = "#login";

        location.hash = routePath;
    },

    // переходим на экран логина
    onLogin: function() {
        this.baseView = new LoginView();

        this.baseView.appendTo(this.content);
        this.baseView.render();
    },

    // убиваем текущую вьюху
    disposeCurrentView: function() {
        if(this.baseView) {
            this.baseView.dispose();
            this.baseView = null;
        }

        SmotreshkaCore.Dom.Helper.removeAllChild(this.content);
    },

    // переходим на плеера
    onPlay: function() {
        this.disposeCurrentView();

        this.baseView = new PlayView();
        this.baseView.appendTo(this.content);
        this.baseView.render();
    },

    // переходим на плеера
    onVod: function() {
        this.disposeCurrentView();

        this.baseView = new VodView();
        this.baseView.appendTo(this.content);
        this.baseView.render();
    }
};

Smotreshka = new SmotreshkaApp();

SmotreshkaCore.App.initialize();
SmotreshkaCore.App.events.on("before", Smotreshka.onBefore.bind(Smotreshka));
SmotreshkaCore.App.events.on("ready", Smotreshka.onReady.bind(Smotreshka));
SmotreshkaCore.App.start();
