var LoginView = function() {};

LoginView.prototype = new SmotreshkaCore.Views.View();
LoginView.prototype = Object.assign(LoginView.prototype, {
    // IDS
    IDS: {
        form: "auth_form_element",
        submit: "submit_nav",
        login: "login_input",
        password: "password_input",
        errors: "auth_form_errors"
    },

    // текущий id
    id: "login_view",

    // элементы
    elements: {},

    //
    qrCodeTime: 2000,

    // название шаблона
    templateName: "login_form",

    //
    isVirtualKeyboard: false,

    // параметры в шаблоне
    params: {
        timestamp: new Date().getTime(),
        Config: SmotreshkaCore.Config,
        qrUrl: SmotreshkaCore.Store.Models.Auth.prototype.url
    },

    // инициализация вьюхи
    initialize: function (options) {
        //
        this.params.qrUrl = SmotreshkaCore.Store.Models.Auth.prototype.url;

        // инициализация запроса на qr код
        this.initQRCode();

        // иницализируем модель авторизации
        this.initAuthModel();

        this.params.t = SmotreshkaCore.Translate.Helper.getCurrentTexts();

        this.navigator = new SmotreshkaCore.SmartTV.VKeyboardNavigator();

        SmotreshkaCore.SmartTV.NavigatorManager.register(this.id, this.navigator);

        SmotreshkaCore.events.on("SmotreshkaCore:SmartTV:VKeyboard:show", this.onKeyboardShow.bind(this));
        SmotreshkaCore.events.on("SmotreshkaCore:SmartTV:VKeyboard:hiden", this.onKeyboardHide.bind(this));

        SmotreshkaCore.events.on("SmotreshkaCore:SmartTV:Keyboard:keydown:BrowserBack", this.onExitKey.bind(this));
        SmotreshkaCore.events.on("SmotreshkaCore:SmartTV:Keyboard:keydown:Exit", this.onExitKey.bind(this));
    },

    onExitKey: function() {
        if(this.isRendered) {
            SmotreshkaCore.events.trigger("SmotreshkaCore:SmartTV:exit");
        }
    },

    // инициализация QR кода
    initQRCode: function() {
        this.qrcode = new SmotreshkaCore.Store.Models.QR();

        this.qrcode.on("sync", this.syncQRCode.bind(this));

        this.qrInterval = setInterval(this.fetchQRCode.bind(this), this.qrCodeTime);
    },

    //
    initAuthModel: function() {
        this.auth = new SmotreshkaCore.Store.Models.Auth();

        this.auth.on("sync", this.authed.bind(this));
        this.auth.on("error", this.onError.bind(this));
    },

    //
    onKeyboardHide: function() {
        this.isVirtualKeyboard = false;

        SmotreshkaCore.SmartTV.NavigatorManager.enable(this.id);
    },

    onKeyboardShow: function() {
        this.isVirtualKeyboard = true;
    },

    //
    syncQRCode: function() {
        Smotreshka.account.fetch();
    },

    //
    fetchQRCode: function() {
        this.qrcode.fetch();
    },

    //
    onRender: function() {
        this.elements.login = document.getElementById(this.IDS.login);
        this.elements.password = document.getElementById(this.IDS.password);

        this.elements.submit = document.getElementById(this.IDS.submit);
        this.elements.submit.addEventListener("click", this.onSubmit.bind(this));

        this.elements.errors = document.getElementById(this.IDS.errors);
        this.elements.title = document.getElementById("title_auth_form");


        this.navigator.setContainer(this.el);

        SmotreshkaCore.SmartTV.NavigatorManager.enable(this.id);
    },

    //
    onSubmit: function() {
        this.auth.model = {
            email: this.elements.login.value,
            password: this.elements.password.value
        };

        this.auth.save();
    },

    //
    onError: function(xhr) {
        var errors = [],
            i = 0,
            errorsLength,
            liElement,
            fragment = document.createDocumentFragment();

        SmotreshkaCore.Dom.Helper.removeAllChild(this.elements.errors);

        if(xhr.errors) {
            errorsLength = xhr.errors.length;
            i = 0;

            for(i = 0; i < errorsLength; i++) {
                errors.push(xhr.errors[i].msg);
            }
        }

        if(xhr.msg) {
            errors.push(xhr.msg);
        }

        errorsLength = errors.length;
        i = 0;

        for(i = 0; i < errorsLength; i++) {
            liElement = document.createElement("li");
            liElement.textContent = errors[i];
            liElement.classList.add("error");

            fragment.appendChild(liElement);
        }

        this.elements.errors.appendChild(fragment);

        fragment = null;
        liElement = null;
    },

    //
    authed: function() {
        var session = this.auth.model.session;

        SmotreshkaCore.Store.TokenManager.setToken(session);

        Smotreshka.account.fetch();
    },

    // деструктор
    onDispose: function() {
        if(this.qrInterval) {
            clearTimeout(this.qrInterval);
        }

        SmotreshkaCore.events.off("SmotreshkaCore:SmartTV:Keyboard:keydown:", this.exitEventId);
    }
});
